package com.blazinggames.retro.classic.search;

// Blazing Games Imports
import com.blazinggames.urcl.data.Config;
import com.blazinggames.urcl.ui.*;
import com.blazinggames.urcl.text.*;

/**
 * Classic Artillery Battle
 * Copyright (C) 2001, 2003-2004 Blazing Games Inc.
 *
 * handles the text version of the artillery game
 * 
 * ------------------------------------------------
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * @author Billy D. Spelchan
 * @version OSR1.00
 */
public class Artillery 
extends TextDisplay
{
// *********************
// ***** CONSTANTS *****
// *********************

    private static final String VERSION = "1.10.040101";
    
    private static final String INSTRUCTIONS = "07INSTRUCTIONS";
    private static final String MIN_DISTANCE = "07MINDIST";
    private static final String MAX_DISTANCE = "07MAXDIST";
    private static final String GRAVITY = "07GRAVITY";
    private static final String UNITBASE = "07UNITBASE";
    private static final String UNITNAME = "07UNITNAME";
    private static final String MAX_POWER = "07MAXPOWER";
    private static final String RADIUS = "07RADIUS";


// *********************
// ***** VARIABLES *****
// *********************

    private Instructions _instructions;

    // configuration properties
    private int _minDistance;
    private int _maxDistance;
    private int _gravity;
    private int _unitBase;
    private String _unitName;
    private int _maxPower;
    private int _radius;
    
    // Game information
    private int _numPlayers;
    private String _playerNames[];
    private boolean _playerIsHuman[];
    private int _playerPlaces[];

    // AI
    private int _aiLastTarget[];
    private int _aiHighs[];
    private int _aiLows[];
    private int _aiAngle[];

    
// **************************************    
// ***** METHODS - From TextDisplay *****
// **************************************

    public boolean initialize(DisplayHandler dh)
    {
        super.initialize(dh);

    	_minDistance = _config.getIntProperty(MIN_DISTANCE, 100);
    	_maxDistance = _config.getIntProperty(MAX_DISTANCE, 1000);
    	_gravity = _config.getIntProperty(GRAVITY, 10);
    	_unitBase = _config.getIntProperty(UNITBASE, 1);
    	_unitName = _config.getStringProperty(UNITNAME, "meter");
    	_maxPower = _config.getIntProperty(MAX_POWER, 1000);
        _radius = _config.getIntProperty(RADIUS, 10);
        return true;
    }

    public void run()
    {
        String s;
        int choice;
        int cntr;
        int round;
        boolean gameOver;
        
        _numPlayers = 0;        
        printHeader();
        instructions();
        
        do
        {
            // Place game loop here
            gameParameters();
            placePlayers();
            round = 0;
            gameOver = false;
            do
            {
                ++round;
                println();
                println("* ROUND " + round + " *");
                println();
                playerStatus();
                println();
                for (cntr = 0; cntr < _numPlayers; ++cntr)
                {
                    if ( (gameOver == false) && (_playerPlaces[cntr] > 0) )
                    {
                        println(_playerNames[cntr] + " turn");
                        if (_playerIsHuman[cntr])
                        {
                            humanShoot(_playerPlaces[cntr]);
                        }
                        else
                        {
                            computerShoot(cntr);                
                        }
                        gameOver = checkGameOver();
                    }
                }
            }
            while (gameOver == false);
            println();
            println();
            
            if (_canQuit)
            {
                print("Do you want another game?");
                s = input();
                println(s);
                if ((s.startsWith("N")) || (s.startsWith("n")))
                    _displayHandler.displayEvent(this, "SHOW 1");
            }
        }
        while (_active);
    }


// *************************************
// ***** METHODS - Game management *****
// *************************************

    private void printHeader()
    {
        println("*** Artillery Battle ***");
        println("Clasic version release " + VERSION);
        println();
        println("Copyright (c) 2001, 2004");
        println("Blazing Games Inc. All Rights Reserved.");
        println();
    }

    private void instructions()
    {
        print("Do you need instructions?");
        String s = input();
        println(s);
        if ( (s.startsWith("y")) || (s.startsWith("Y")) )
        {
            if (_instructions == null)
            {
                println("Grabbing instructions...");
                _res.addTextGroup(INSTRUCTIONS, _config.getStringProperty(INSTRUCTIONS, "nuke.txt"));
                _instructions = new Instructions(_res.getTextGroup(INSTRUCTIONS));
                println("Done.");
                println();
            }
            _instructions.showInstructions(this);
        }
    }

    private void gameParameters()
    {
        String s;
        
        if (_numPlayers > 1)
        {
            println("Keep same settings? ");
            s = input();
            if ( (s.startsWith("y")) || (s.startsWith("Y")) )
                return;
        }
        
        do
        {
            print("How many players (2 to 9)? ");
            s = input();
            _numPlayers = _config.parseInt(s, 0);
            println("" + _numPlayers);
        }
        while ( (_numPlayers < 2) || (_numPlayers > 9) );
        
        int numAIs = 0;
        _playerNames = new String[_numPlayers];
        _playerIsHuman = new boolean[_numPlayers];
        for (int cntr = 0; cntr < _numPlayers; ++cntr)
        {
            print("is player " + (cntr + 1) + " human? ");
            s = input();
            println(s);
            _playerIsHuman[cntr] = ( (s.startsWith("y")) || (s.startsWith("Y")) );
            if (_playerIsHuman[cntr])
            {
                print("Name: ");
                _playerNames[cntr] = input();
                println(_playerNames[cntr]);
            }
            else
            {
                ++numAIs;
                _playerNames[cntr] = "Computer " + numAIs;
                println("Being named " + _playerNames[cntr]);
            }
        }
    }

    private void placePlayers()
    {
        _playerPlaces =  new int[_numPlayers];
        _aiLastTarget = new int[_numPlayers];
    	_aiHighs = new int[_numPlayers];
    	_aiLows = new int[_numPlayers];
        _aiAngle = new int[_numPlayers];

        int lastPlayer = 0;
        for (int cntr = 0; cntr < _numPlayers; ++cntr)
        {
            _playerPlaces[cntr] = rand(_minDistance, _maxDistance) + lastPlayer;
            lastPlayer = _playerPlaces[cntr];
            _aiLastTarget[cntr] = -1;
            _aiHighs[cntr] = 0;
            _aiLows[cntr] = 0;
        }
    }
    
    private void playerStatus()
    {
        for (int cntr = 0; cntr < _numPlayers; ++cntr)
        {
            if (_playerPlaces[cntr] > 0)
                println(_playerNames[cntr] + " is at " + _playerPlaces[cntr]);
        }
    }

    private void humanShoot(int loc)
    {
        String s;
        int angle;
        int power;
        
        do
        {
            print("Angle (5 - 175)? ");
            s = input();
            println(s);
            angle = _config.parseInt(s, 0);
        }
        while ( (angle < 5) || (angle > 175) );
        
        do
        {
            print("Power (1 - " + _maxPower + ")? ");
            s = input();
            println(s);
            power = _config.parseInt(s, 0);
        }
        while ( (power < 1) || (power > _maxPower) );
        
        print("Show path? ");
        s = input().toUpperCase();
        println(s);
        processShot(loc, angle, power, s.startsWith("Y"));
    }
    
    private void computerShoot(int id)
    {
        boolean needTarget = true;
        int target;
        int power = 1;
        int angle = 1;
        
        if (_aiLastTarget[id] > -1)
            needTarget = (_playerPlaces[_aiLastTarget[id]] <= 0);
        if (needTarget)
        {
            do
            {
                target = rand(0, _numPlayers - 1);
            }
            while ( (target == id) || (_playerPlaces[target] <= 0) );
            _aiLastTarget[id] = target;
            _aiHighs[id] = 0;
            _aiLows[id] = 0;
            _displayHandler.log("AI " + id + " targetting " + target);
            if (_playerPlaces[id] < _playerPlaces[target])
                angle = rand(5, 89);
            else
                angle = rand(91, 175);
            _aiAngle[id] = angle;
            power = rand(50, 200);                    
        }
        else
        {
            target = _aiLastTarget[id];
            angle = _aiAngle[id];
            if (_aiHighs[id] > 0)
            {
                power = rand(_aiLows[id], _aiHighs[id]);
            }
            else
            {
                power = rand(_aiLows[id], _aiLows[id] + 100);                    
            }
        }
        
        int hit = processShot(_playerPlaces[id], angle, power, false);
        if (angle > 90)
        {
            if (hit < _playerPlaces[target])
            {
                _aiHighs[id] = power;
            }
            else
            {
                _aiLows[id] = power;
            }
        }
        else
        {
            if (hit < _playerPlaces[target])
            {
                _aiLows[id] = power;
            }
            else
            {
                _aiHighs[id] = power;
            }
        }
    }
    
    private int processShot(int loc, int angle, int power, boolean showShot)
    {
        double rad = (double)angle * Math.PI / 180.0;
_displayHandler.log("angle " + angle + " is radian " + rad);
        double rise = Math.sin(rad) * power;
        double run = Math.cos(rad) * power;
_displayHandler.log("rise = " + rise + "run = " + run);
        int x = loc;
        int y = 0;
        int oldX, oldY, distY;
        do 
        {
            oldX = x;
            oldY = y;
            x += (int)run;
            y += (int)rise;
            distY = Math.abs((int)rise);
            rise -= _gravity;
            if (y <= 0)
            {
                int hitRun = (int)run * oldY / (distY + oldY);
                x = oldX + hitRun;
                y = 0;
            }
            if (showShot)
            {
                println("Bullet at " + x + "," + y);
            }
        }
        while (y > 0);
        int lowRange = Math.max(1, x - _radius);
        int highRange = Math.max(1, x + _radius);
        println("impact area = " + lowRange + " to " + highRange);
        
        for (int cntr = 0; cntr < _numPlayers; ++cntr)
        {
            if ( (_playerPlaces[cntr] >= lowRange) && (_playerPlaces[cntr] <= highRange) )
            {
                println("Cannon destroyed: " + _playerNames[cntr]);
                _playerPlaces[cntr] = 0;
            }
        }
        
        return x;
    }
    
    private boolean checkGameOver()
    {
        int liveCount = 0;
        int humanCount = 0;
        int lastAlive = -1;
        
        for (int cntr = 0; cntr < _numPlayers; ++cntr)
        {
            if (_playerPlaces[cntr] > 0)
            {
                if (_playerIsHuman[cntr])
                    ++humanCount;
                ++liveCount;
                lastAlive = cntr;
            }
        }
        
        if (humanCount == 0)
        {
            println("The computer has won!");
            return true;
        }
        else if (liveCount == 0)
        {
            println("Game ends in a draw!");
            return true;
        }
        else if (liveCount == 1)
        {
            println("Player " + _playerNames[lastAlive] + " WON!");
            return true;
        }
        
        return false;
    }
}
